import java.awt.Button;
import java.awt.Choice;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.Label;
import java.awt.Panel;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

//last modified 2007.04.09
//2007.04.16: PROBLEM: resultLabel is not redrawn in applet (only after manual resize)

public class GUI extends Panel implements ActionListener
{
  private static final String FONT_NAME="arial";
  private static final int FONT_SIZE=16;

  private final Panel north=new Panel();
  private final Panel center=new Panel();

  private final Label normalTime=new Label("- ms");
  private final Label optiTime=new Label("- ms");
  private final Label resultLabel=new Label("???");

  private final Button startTestButton=new Button(" Run Test ");

  private final Component topMostComponent; //component which includes all other
  // if started as application, this-object the Applet-object otherwise

  private OptiTest test;

  public GUI(final OptiTest test, final Component app)
  {
    if(null!=app)
      topMostComponent=app;
    else
      topMostComponent=this;

    //set-font
    this.setFont(new Font(FONT_NAME, Font.PLAIN , FONT_SIZE));
    startTestButton.setFont(new Font(FONT_NAME, Font.BOLD , FONT_SIZE));

    this.setLayout(new GridLayout(2,1)); //North center south


    if(null==test)//null means no display a choice in the GUI
    {
      //!!! To be implemented (ask main for a list of all classes)
      final Choice choices= new Choice();
      choices.add("Wahl 1");
      choices.add("Wahl 2");
      choices.add("Wahl 3");
      choices.add("Wahl 4");
      choices.add("Wahl 5");

      choices.setFont(new Font(FONT_NAME, Font.PLAIN , FONT_SIZE));

//!!! add a Listener to get choice and then set the test variable

      north.add(choices);
    }//endif
    else
    {
      this.test=test;
      north.add(new Label(test.getTestname()));
    }//end-else

    startTestButton.addActionListener((ActionListener)this);
    north.add(startTestButton);

    {
      final Label l=new Label("Normal:");
      l.setFont(new Font(FONT_NAME, Font.BOLD , FONT_SIZE));
      center.add(l);
    }//end-block
    center.add(normalTime);

    {
      final Label l=new Label("Optimised:");
      l.setFont(new Font(FONT_NAME, Font.BOLD , FONT_SIZE));
      center.add(l);
    }//end-block
    center.add(optiTime);

    resultLabel.setFont(new Font(FONT_NAME, Font.BOLD , FONT_SIZE));
    center.add(resultLabel);


    this.add(north);
    this.add(center);

  }//end-method

  public void init()
  {
  }

  private boolean firstCall=true;
  public void actionPerformed(final ActionEvent e)
  {
    startTestButton.setEnabled(false); //deactivate the button

//    System.err.println("actionPerformed: "+e);//                       //DEBUG

    optiTime.setText("- ms");
    normalTime.setText("- ms");
    this.validate(); //casues a relayout to all Components

    if(firstCall)
    { firstCall=false;
      test.findAndSetNumberOfRuns(1000);
    }

    final int nTime=test.runUnoptimisedVersion();
    normalTime.setText(nTime+"ms");

    final int oTime=test.runOptimisedVersion();
    optiTime.setText(oTime+"ms");

    if(nTime>=oTime)
    {
      resultLabel.setForeground(Color.GREEN);
      resultLabel.setText(((10000-((10000*oTime)/nTime))/100.)+"% faster");
    }//endif
    else
    {
      resultLabel.setForeground(Color.RED);
      resultLabel.setText(((-10000+((10000*oTime)/nTime))/100.)+"% slower");
    }//end-else

    topMostComponent.validate(); //casues a relayout to all Components

    startTestButton.setEnabled(true); // activate the button again
  }//end-method

}//end GUI-class
